#!/usr/bin/env python3
#coding=utf-8
"""..."""
__author__ = 'Simon J. Greenhill <simon@simon.net.nz>'
__copyright__ = 'Copyright (c) 2016 Simon J. Greenhill'
__license__ = 'New-style BSD'

import unittest

from abvdget.abvd_download import parse_args as abvd_download_parse_args


class TestABVDDownload_ParseArgs(unittest.TestCase):
    def test_two(self):
        self.assertEqual(
            abvd_download_parse_args(['austronesian', '10']),
            ('austronesian', 10, None)
        )
        
    def test_three(self):
        self.assertEqual(
            abvd_download_parse_args(['austronesian', '10', '-o', 'out.json']),
            ('austronesian', 10, 'out.json')
        )
        
    def test_invalid_database(self):
        with self.assertRaises(SystemExit) as e:
            abvd_download_parse_args(['wals', '10'])
            
    def test_invalid_language_id(self):
        with self.assertRaises(SystemExit) as e:
            abvd_download_parse_args(['austronesian', 'maori'])
    


#!/usr/bin/env python3
# coding=utf-8

import json
import codecs
from xml.dom import minidom
from functools import lru_cache

from abvdget.tools import slugify

try:
    import requests
except ImportError:  # pragma: no cover
    pass

URL = "http://abvd.shh.mpg.de/utils/save/?type=xml&section=%(db)s&language=%(id)d"

XMLTEMPLATE = """
<?xml version="1.0" encoding="utf-8"?>
<abvd>
    %s
</abvd>
""".lstrip().rstrip()

DATABASES = [
    'austronesian',
    'bantu',
    'mayan',
    'utoaztecan',
]

DEAD_LANGUAGES = {
    'bantu': [],
    'mayan': [],
    'utoaztecan': [],
    'austronesian': [
        261,  # Futuna-Aniwa
        874,  # proto-philippines
    ],
}


class DeadLanguageError(ValueError):
    pass


class InvalidLanguageError(ValueError):
    pass


class Record(object):
    """Data Class for Lexical Records."""
    def __init__(self, ID=None, LID=None, WID=None, Language=None, Word=None, Item=None, Annotation=None, Loan=None, Cognacy=None):
        self.ID = ID
        self.LID = LID
        self.WID = WID
        self.Language = Language
        self.Word = Word
        self.Item = Item
        self.Annotation = Annotation
        self.Loan = Loan
        self.Cognacy = Cognacy

    def __repr__(self):
        return "<Record %s - %s - %s - %s>" % (
            self.ID, self.Language, self.Word, self.Item
        )

    @property
    def is_loan(self):
        """Is this lexeme a loan."""
        if self.Loan is None:
            return False
        elif self.Loan in (False, ""):
            return False
        elif self.Loan is True:
            return True
        else:
            return True

    def get_taxon(self):
        if self.LID is None:
            return self.Language
        else:
            return "%s_%d" % (slugify(self.Language), self.LID)



class Downloader(object):
    
    databases = DATABASES
    
    def __init__(self, database, url=URL):
        if database not in self.databases:
            raise ValueError("Unknown Database: %s" % database)
        self.database = database
        self.data = None
        self.url = URL
        
    def make_url(self, language_id):
        try:
            language_id = int(language_id)
        except:
            raise TypeError("language_id needs to be an integer")
        
        return self.url % {'db': self.database, 'id': language_id}
    
    @lru_cache(maxsize=2000)
    def get(self, language_id):  # pragma: no cover
        language_id = self.is_valid_language(language_id)
        req = requests.get(self.make_url(language_id))
        
        # fail on no content
        if len(req.content) == 0 or req.content.strip() == 'null':
            raise InvalidLanguageError("Language %d does not exist" % language_id)
            
        try:
            content = req.content.decode('utf8')
        except:
            raise
            
        self.data = Parser().parse(content)
        return self.data
    
    def write(self, filename, content=None):  # pragma: no cover
        with codecs.open(filename, 'w', encoding="utf8") as handle:
            handle.write(json.dumps(
                content if content else self.data,
                sort_keys=True, indent=2,
                separators=(',', ': '), ensure_ascii=False
            ))
    
    def is_valid_language(self, language_id):
        if not isinstance(language_id, int):
            raise InvalidLanguageError("Language id must be an integer")
        if language_id in DEAD_LANGUAGES[self.database]:
            raise DeadLanguageError("Language %d has been removed" % language_id)
        return language_id
        
    def get_to_file(self, language_id, filename):  # pragma: no cover
        self.write(filename, self.get(language_id))


class Parser(object):
    def is_language(self, adict):
        expected = [
            'id', 'checkedby', 'language', 'classification', 'author',
            'silcode', 'glottocode', 'notes', 'typedby', 'problems'
        ]
        return all([e in adict.keys() for e in expected])

    def is_lexicon(self, adict):
        expected = [
            'id', 'word_id', 'word',
            'item', 'annotation', 'loan', 'cognacy', 'pmpcognacy',
        ]
        return all([e in adict.keys() for e in expected])
    
    def is_lexicon_2(self, adict):
        expected = [
            'id', 'word_id', 'word',
            'source_id', 'source',
            'item', 'annotation', 'loan', 'cognacy',
        ]
        return all([e in adict.keys() for e in expected])
    
    def is_location(self, adict):
        expected = ['latitude', 'longitude']
        return all([e in adict.keys() for e in expected])

    def parse(self, content):
        xml = minidom.parseString(XMLTEMPLATE % content)
        entities = {'language': None, 'lexicon': [], 'location': None}
        for node in xml.getElementsByTagName('record'):
            content = {}
            for child in node.childNodes:
                # skip text nodes (which are whitespace indentation)
                if child.nodeType == child.TEXT_NODE:
                    continue

                tag = child.tagName
                assert tag not in content

                try:
                    data = child.firstChild.data
                except AttributeError:
                    # no data e.g. <x></x>
                    data = None

                content[tag] = data
            
            if self.is_language(content):
                if entities['language'] is not None:
                    raise ValueError("Encountered Duplicate Language Record")
                entities['language'] = content
            elif self.is_lexicon(content):
                entities['lexicon'].append(content)
            elif self.is_lexicon_2(content):
                entities['lexicon'].append(content)
            elif self.is_location(content):
                entities['location'] = content
            else:
                raise ValueError("Unknown Record Type: %r" % content)

        # check
        if entities['language'] is None:
            raise ValueError("No Language Record Found!")
        if len(entities['lexicon']) == 0:
            raise ValueError("No Lexical Records Found!")

        return entities


class ABVDatabase(object):
    def __init__(self, files=None):
        self.files = {}
        self._lexicon_by_file = {}
        self.records = None
        
        if files:
            for f in files:
                self.load(f)
        
    def load(self, filename):
        with codecs.open(filename, 'r', encoding="utf8") as handle:
            self.files[filename] = json.load(handle, encoding="utf8")
    
    def get_details(self, filename):
        d = self.files[filename].get('language')
        if d is None:
            raise ValueError("Data for %s not loaded" % filename)
        d['filename'] = filename
        return d

    def get_location(self, filename):
        return self.files[filename]['location']

    def get_lexicon(self, filename):
        if filename not in self._lexicon_by_file:
            self._lexicon_by_file[filename] = []
            d = self.get_details(filename)
            for r in self.files[filename]['lexicon']:
                self._lexicon_by_file[filename].append(self.to_record(d, r))
        yield from self._lexicon_by_file[filename]
        
    @lru_cache(maxsize=2000)
    def get_nlexemes(self, filename):
        return len(list(self.get_lexicon(filename)))

    @lru_cache(maxsize=2000)
    def get_ncognates(self, filename):
        return len([
            r for r in self.get_lexicon(filename)
            if r.Cognacy is not None
        ])
    
    @lru_cache(maxsize=2000)
    def get_slug_for(self, taxon, id):
        return "%s_%s" % (slugify(taxon), id)
    
    def to_record(self, details, entry):
        return Record(
            LID=int(details['id']),
            ID=int(entry['id']),
            WID=int(entry['word_id']),
            Language=details['language'],
            Word=entry['word'],
            Item=entry['item'],
            Annotation=entry['annotation'],
            Loan=entry['loan'],
            Cognacy=entry['cognacy']
        )
    
    def process(self):
        self.records = []
        for filename in self.files:
            d = self.get_details(filename)
            self.records.extend(self.get_lexicon(filename))
        return self.records
    
    def save_details(self, filename):
        def denone(v):
            return '' if v is None else v

        def check_tabs(v):
            assert "\t" not in v

        def fmt_loc(v):
            return "%0.4f" % float(v) if v != '-' else v
        
        with codecs.open(filename, 'w', encoding="utf8") as out:
            out.write("\t".join([
                "ID", "ISO", "Glottocode", "Language", "Slug", "NLexemes",
                "NCognates","Author", "Latitude", "Longitude", "Classification"
            ]))
            out.write("\n")
            for f in self.files:
                lang = self.get_details(f)
                loc = self.get_location(f)
                loc = {'longitude': '-', 'latitude': '-'} if loc is None else loc
                taxon = self.get_slug_for(lang['language'], lang['id'])
                line = [
                    lang['id'],
                    denone(lang['silcode']),
                    denone(lang['glottocode']),
                    lang['language'],
                    taxon,
                    '%d' % self.get_nlexemes(f),
                    '%d' % self.get_ncognates(f),
                    denone(lang['author']),
                    fmt_loc(loc['latitude']),
                    fmt_loc(loc['longitude']),
                    denone(lang['classification']),
                ]
                try:
                    [check_tabs(v) for v in line]
                except (AssertionError, TypeError):  # pragma: no cover
                    print("ERROR", line)
                    raise
                out.write("\t".join(line))
                out.write("\n")
        return



import os
import unittest
import tempfile

from abvdget import ABVDatabase, Record

TESTDATA = os.path.join(os.path.dirname(__file__), 'nengone.json')

EXPECTED = {
    99: {
        "LID": 99,
        "Annotation": "arm and hand",
        "Cognacy": '1',
        "Item": "nin",
        "Loan": None,
        "Word": "hand",
        "WID": 1
    }, 
    93340: {
        "LID": 99,
        "Annotation": None,
        "Cognacy": '13',
        "Item": "iñtërnâtiônàlizætiøn",
        "Loan": None,
        "Word": "leg/foot",
        "WID": 4,
    },
    90697: {
        "LID": 99,
        "Annotation": None,
        "Cognacy": None,
        "Item": "kaka",
        "Loan": None,
        "Word": "to eat",
        "WID": 37
    },
    70785: {
        "LID": 99,
        "Annotation": None,
        "Cognacy": '1',
        "Item": "tini",
        "Loan": None,
        "Word": "Three",
        "WID": 199
    }
}


class TestABVD(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.abvd = ABVDatabase(files=[TESTDATA])
    
    def test_load(self):
        assert TESTDATA in self.abvd.files
    
    def test_get_details(self):
        d = self.abvd.get_details(TESTDATA)
        assert d['id'] == '99'
        assert d['language'] == 'Nengone'
        assert d['silcode'] == 'nen'
        assert d['glottocode'] == 'neng1238'
    
    def test_get_details_injects_filename(self):
        d = self.abvd.get_details(TESTDATA)
        assert d.get('filename') == TESTDATA
    
    def test_get_location(self):
        d = self.abvd.get_location(TESTDATA)
        assert d['latitude'] == "-21.53484700204878876661"
        assert d['longitude'] == "167.98095703125000000000"
    
    def test_get_nlexemes(self):
        assert self.abvd.get_nlexemes(TESTDATA) == 4

    def test_get_ncognates(self):
        assert self.abvd.get_ncognates(TESTDATA) == 3
    
    def test_process(self):
        for r in self.abvd.process():
            assert r.ID in EXPECTED
            for k in EXPECTED[r.ID]:
                self.assertEqual(EXPECTED[r.ID][k], getattr(r, k))
    
    def test_get_slug_for(self):
        self.assertEqual(self.abvd.get_slug_for('Nengone', '99'), 'Nengone_99')
    
    def test_save_details(self):
        with tempfile.NamedTemporaryFile() as out:
            self.abvd.save_details(out.name)
            out.seek(0)
            content = out.readlines()
        assert len(content) == 2  # two lines, header and Nengone



#!/usr/bin/env python3
#coding=utf-8
"""..."""
__author__ = 'Simon J. Greenhill <simon@simon.net.nz>'
__copyright__ = 'Copyright (c) 2016 Simon J. Greenhill'
__license__ = 'New-style BSD'

import os
import codecs
import unittest
from xml.dom import minidom

from abvdget import Parser

TESTDATA = os.path.join(os.path.dirname(__file__), 'nengone.xml')

LEX = {
    'annotation': '',
    'pmpcognacy': '',
    'cognacy': '',
    'word': '',
    'id': '',
    'loan': '',
    'word_id': '',
    'item': ''
}
LNG = {
    'notes': None,
    'typedby': 'Penny Keenan',
    'classification': 'Austronesian, Malayo-Polynesian, Central-Eastern Malayo-Polynesian, Eastern Malayo-Polynesian, Oceanic, Central-Eastern Oceanic, Remote Oceanic, Loyalty Islands',
    'checkedby': 'Simon Greenhill',
    'problems': None,
    'silcode': 'nen',
    'glottocode': 'neng1238',
    'author': 'Blust',
    'language': 'Nengone',
    'id': '99'
}

LOC = {
    'latitude': '-21.53484700204878876661',
    'longitude': '167.98095703125000000000',
}



XML_DUPLICATE_LANGUAGE = """
<record>
    <id>99</id>
    <language>Nengone</language>
    <author>Blust</author>
    <silcode>nen</silcode>
    <glottocode>neng1238</glottocode>
    <notes></notes>
    <problems></problems>
    <classification></classification>
    <typedby>Penny Keenan</typedby>
    <checkedby>Simon Greenhill</checkedby>
</record>

<record>
    <id>99</id>
    <language>Nengone</language>
    <author>Blust</author>
    <silcode>nen</silcode>
    <glottocode>neng1238</glottocode>
    <notes></notes>
    <problems></problems>
    <classification></classification>
    <typedby>Penny Keenan</typedby>
    <checkedby>Simon Greenhill</checkedby>
</record>

<record>
    <id>93340</id>
    <word_id>4</word_id>
    <word>leg/foot</word>
    <item>iñtërnâtiônàlizætiøn</item>
    <annotation></annotation>
    <loan></loan>
    <cognacy>13</cognacy>
    <pmpcognacy></pmpcognacy>
</record>

"""

XML_UNKNOWN_RECORD = """
<record>
    <id>99</id>
    <language>Nengone</language>
    <author>Blust</author>
    <silcode>nen</silcode>
    <glottocode>neng1238</glottocode>
    <notes></notes>
    <problems></problems>
    <classification></classification>
    <typedby>Penny Keenan</typedby>
    <checkedby>Simon Greenhill</checkedby>
</record>

<record>
    <id>93340</id>
    <word_id>4</word_id>
    <word>leg/foot</word>
    <item>iñtërnâtiônàlizætiøn</item>
    <annotation></annotation>
    <loan></loan>
    <cognacy>13</cognacy>
    <pmpcognacy></pmpcognacy>
</record>

<record>
    <something>y</something>
</record>
"""

XML_NO_LANGUAGE = """
<record>
    <id>93340</id>
    <word_id>4</word_id>
    <word>leg/foot</word>
    <item>iñtërnâtiônàlizætiøn</item>
    <annotation></annotation>
    <loan></loan>
    <cognacy>13</cognacy>
    <pmpcognacy></pmpcognacy>
</record>
"""

XML_NO_LEXICON = """
<record>
    <id>99</id>
    <language>Nengone</language>
    <author>Blust</author>
    <silcode>nen</silcode>
    <glottocode>neng1238</glottocode>
    <notes></notes>
    <problems></problems>
    <classification></classification>
    <typedby>Penny Keenan</typedby>
    <checkedby>Simon Greenhill</checkedby>
</record>
"""


class Test_Parser(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        with codecs.open(TESTDATA, 'r') as handle:
            content = handle.read()
        cls.result = Parser().parse(content)
    
    def test_is_lexicon(self):
        self.assertFalse(Parser().is_lexicon(LNG))
        self.assertFalse(Parser().is_lexicon(LOC))
        self.assertTrue(Parser().is_lexicon(LEX))
        
    def test_is_language(self):
        self.assertTrue(Parser().is_language(LNG))
        self.assertFalse(Parser().is_language(LOC))
        self.assertFalse(Parser().is_language(LEX))
    
    def test_is_location(self):
        self.assertFalse(Parser().is_location(LNG))
        self.assertTrue(Parser().is_location(LOC))
        self.assertFalse(Parser().is_location(LEX))
        
    def test_parse_language(self):
        for k in LNG:
            self.assertEqual(LNG[k], self.result['language'][k])
    
    def test_parse_location(self):
        for k in LOC:
            self.assertEqual(LOC[k], self.result['location'][k])
    
    def test_lexicon_count(self):
        self.assertEqual(len(self.result['lexicon']), 4)
    
    def test_lexicon_1(self):
        lex = self.result['lexicon'][0]
        assert lex['id'] == '99'
        assert lex['word'] == 'hand'
        assert lex['item'] == 'nin'
        assert lex['annotation'] == 'arm and hand'
        assert lex['cognacy'] == '1'
    
    def test_lexicon_2(self):
        o = {
            "id": "99",
            "word": "hand",
            "word_id": "1",
            "source": 'greenhill2011',
            "source_id": '1',
            "annotation": "arm and hand",
            "cognacy": "1",
            "item": "nin",
            "loan": None,
        }
        assert Parser().is_lexicon_2(o)
    
    def test_empty_cells_are_none(self):
        lex = self.result['lexicon'][0]
        assert lex['id'] == '99'
        assert lex['loan'] is None
    
    def test_utf8(self):
        expected = "iñtërnâtiônàlizætiøn"
        lex = self.result['lexicon'][1]
        assert lex['item'] == expected
    
    def test_duplicate_language(self):
        with self.assertRaises(ValueError):
            Parser().parse(XML_DUPLICATE_LANGUAGE)
        
    def test_unknown_object_type(self):
        with self.assertRaises(ValueError):
            Parser().parse(XML_UNKNOWN_RECORD)

    def test_no_language(self):
        with self.assertRaises(ValueError):
            Parser().parse(XML_NO_LANGUAGE)

    def test_no_lexicon(self):
        with self.assertRaises(ValueError):
            Parser().parse(XML_NO_LEXICON)



#!/usr/bin/env python3
#coding=utf-8
"""..."""
__author__ = 'Simon J. Greenhill <simon@simon.net.nz>'
__copyright__ = 'Copyright (c) 2016 Simon J. Greenhill'
__license__ = 'New-style BSD'

import os
import unittest
from abvdget import Downloader, DeadLanguageError, InvalidLanguageError


class Test_Downloader(unittest.TestCase):
    def test_error_on_invalid_database(self):
        with self.assertRaises(ValueError):
            Downloader('uralic')
    
    def test_make_url_error_on_noniteger(self):
        with self.assertRaises(TypeError):
            Downloader('bantu').make_url('A')
    
    def test_make_url(self):
        url = Downloader('bantu').make_url('999')
        assert url.endswith('999')
        assert 'bantu' in url
    
    def test_is_valid_language(self):
        d = Downloader('austronesian')
        with self.assertRaises(InvalidLanguageError):
            d.is_valid_language("A")
        with self.assertRaises(InvalidLanguageError):
            d.is_valid_language("Maori")
        with self.assertRaises(DeadLanguageError):
            d.is_valid_language(261)
        # but 261 is only invalid in austronesian
        assert Downloader("bantu").is_valid_language(261)



#!/usr/bin/env python3
#coding=utf-8
import sys
import argparse

from .ABVD import DATABASES, Downloader
from . import __version__

def parse_args(args):
    """
    Parses command line arguments

    Returns a tuple of (inputfile, method, outputfile)
    """
    parser = argparse.ArgumentParser(description='Downloads data from the ABVD')
    parser.add_argument('--version', action='version', version='%s' % __version__)
    parser.add_argument("database", help="database", choices=DATABASES)
    parser.add_argument("language", help="language", type=int)
    parser.add_argument(
        '-o', "--output", dest='output', default=None,
        help="output file", action='store'
    )
    args = parser.parse_args(args)
    return (args.database, args.language, args.output)


def main(args=None):  # pragma: no cover
    if args is None:
        args = sys.argv[1:]
    database, language, outfile = parse_args(args)
    
    d = Downloader(database)
    d.get(language)
    if outfile:
        d.write(outfile)
    else:
        print(d.data)



import unittest

from abvdget import Record

class TestRecord(unittest.TestCase):
    
    def test_simple(self):
        r = Record(ID=1, WID=2, LID=3, Language='English', Word='Hand', Item='hand', Annotation='?', Cognacy=None, Loan="L")
        assert r.ID == 1
        assert r.WID == 2
        assert r.LID == 3
        assert r.Language == 'English'
        assert r.Word == 'Hand'
        assert r.Item == 'hand'
        assert r.Annotation == '?'
        assert r.Loan == "L"
        assert r.Cognacy == None
    
    def test_is_loan(self):
        assert not Record(Loan="").is_loan
        assert not Record(Loan=None).is_loan
        
        assert Record(Loan="L").is_loan
        assert Record(Loan="English").is_loan
        assert Record(Loan=True).is_loan
    
    def test_get_taxon(self):
        r = Record(ID=1, WID=2, LID=3, Language='English', Word='Hand', Item='hand', Annotation='?', Cognacy=None, Loan="L")
        assert r.get_taxon() == "English_3"
        r = Record(ID=1, WID=2, LID=None, Language='English', Word='Hand', Item='hand', Annotation='?', Cognacy=None, Loan="L")
        assert r.get_taxon() == "English"
    
    def test_get_taxon_spaces(self):
       r = Record(ID=1, WID=2, LID=583, Language='Arosi (Oneibia Village)', Word='Hand', Item='hand', Annotation='?', Cognacy=None, Loan="L")
       assert r.get_taxon() == "Arosi_Oneibia_Village_583"
      
    def test_get_taxon_unicode(self):
       r = Record(ID=1, WID=2, LID=1502, Language='Wamoaŋ', Word='Hand', Item='hand', Annotation='?', Cognacy=None, Loan="L")
       assert r.get_taxon() == "Wamoang_1502"
    
    def test_repr(self):
       r = Record(ID=1, WID=2, LID=3, Language='English', Word='Hand', Item='hand', Annotation='?', Cognacy=None, Loan="L")
       repr(r)
      



import unicodedata

def clean(var):
    """Removes tabs, newlines and trailing whitespace"""
    if var is None:
        return ''
    return var.replace("\t", "").replace("\n", "").strip()

def slugify(var):
    remove = ["(", ")", ";", "?", '’', "'", ".", ",", ':', "‘",]
    replace = [
        (" - ", '_'), (" ", "_"), ("ŋ", "ng"), ('ʝ', "j"),
        ('ɛ', 'e'), ('ʃ', 'sh'), ('ø', 'Y'), ('ɲ', 'nj'),
    ]
    var = var.split("[")[0].strip()
    var = var.split("/")[0].strip()
    var = unicodedata.normalize('NFKD', var)
    var = "".join([c for c in var if not unicodedata.combining(c)])
    var = var.replace("ß", "V")  # do this before casefolding
    var = var.casefold()
    for r in remove:
        var = var.replace(r, "")
    for r in replace:
        var = var.replace(*r)
    var = var.title()
    return var



#!/usr/bin/env python
from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

from abvdget import __version__

setup(
    name='abvdget',
    version=__version__,
    description="abvdget - download data from the Austronesian Basic Vocabulary Database",
    url='',
    author='Simon J. Greenhill',
    author_email='simon@simon.net.nz',
    license='BSD',
    zip_safe=True,
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: BSD License",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "Topic :: Software Development :: Libraries :: Python Modules",
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    keywords='austronesian data API',
    packages=find_packages(),
    install_requires=['requests', ],
    entry_points={
        'console_scripts': [
            'abvd_download = abvdget.abvd_download:main'
        ],
    },
    test_suite="abvdget.tests",
)



#!/usr/bin/env python3
#coding=utf-8
"""..."""
__author__ = 'Simon J. Greenhill <simon@simon.net.nz>'
__copyright__ = 'Copyright (c) 2016 Simon J. Greenhill'
__license__ = 'New-style BSD'

import unittest

from abvdget.tools import clean, slugify


class TestClean(unittest.TestCase):
    def test_none(self):
        self.assertEqual(clean(None), '')
        
    def test_newline(self):
        self.assertEqual(clean("la\nng"), 'lang')
        
    def test_tab(self):
        self.assertEqual(clean("la\tng"), 'lang')


class Test_Slugify(unittest.TestCase):
    def test_brackets(self):
        self.assertEqual(slugify('Banggai (W.dialect)'), 'Banggai_Wdialect')

    def test_square_brackets(self):
        self.assertEqual(slugify('Buru [Namrole Bay]'), 'Buru')

    def test_dash(self):
        self.assertEqual(slugify('Aklanon - Bisayan'), 'Aklanon_Bisayan')

    def test_accents(self):
        self.assertEqual(slugify('Gimán'), 'Giman')
        self.assertEqual(slugify('Hanunóo'), 'Hanunoo')

    def test_colon(self):
        self.assertEqual(slugify('Kakiduge:n Ilongot'), 'Kakidugen_Ilongot')

    def test_slash(self):
        self.assertEqual(slugify('Angkola / Mandailin'), 'Angkola')
    
    def test_apostrophe(self):
        self.assertEqual(slugify('V’ënen Taut'), 'Venen_Taut')
        
    def test_B(self):
        self.assertEqual(slugify('ßatarobu'), 'Vatarobu')
    
    def test_ng(self):
        self.assertEqual(slugify("Siŋorakai"), "Singorakai")
    
    def test_capital_NG(self):
        self.assertEqual(slugify("Bawah Ŋgusumbatu"), "Bawah_Nggusumbatu")

    def test_j(self):
        self.assertEqual(slugify('xʝalasu'), 'Xjalasu')
        self.assertEqual(slugify('ʝalasu'), 'Jalasu')
    
    def test_e(self):
        self.assertEqual(slugify('Moaekɛ'), 'Moaeke')
    
    def test_palatal_nasal(self):
        self.assertEqual(slugify('Ɲua'), 'Njua')
    
    def test_another_apostrophe(self):
        self.assertEqual(slugify('‘Aragur'), 'Aragur')
